(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    define([], factory);
  } else if (typeof exports === 'object' && typeof module !== "undefined") {
    module.exports = factory();
  } else {
    root.checkboxTree = factory();
  }
}(this, function () {
    var checkboxTree = function (node, settings) {
      var defaultLevel = {
        children: 'children',
        checkedProperty: 'flag',
        name: 'name',
        id: 'id',
        parentId: 'parent_id'
      }
      this.node = node;
      this.data = settings.data;
      var level = this.extend(settings.level, defaultLevel);
      console.log(level)
      this.childrenSign = level.children;
      this.checkedProperty = level.checkedProperty;
      this.nameSign = level.name;
      this.idSign = level.id;
      this.parentIdSign = level.parentId;
      this.changeHandler =  settings.changeHandler;
      this.created = settings.created;
      this.dataMap = {};
      this.selectedData = [];
      this.getSelectedNodes = this._getSelectedNodes
      this.init();
    }
    checkboxTree.prototype = {
      init: function () {
        this.node.innerHTML = this.getTree(this.getTreeDom(this.data))
        var treeLayout = this.node.querySelector('.checkbox-tree')
        treeLayout.addEventListener('click', this.clickHandler.bind(this))
        if (this.created) this.created()
      },
      getTreeDom: function (nodeData) {
        var domStr = ''
        var empty = true
        nodeData.forEach(function (value) {
          this.dataMap[value['id']] = value
          if (value[this.childrenSign] && value[this.childrenSign].length) {
            empty = true
            domStr = domStr 
              + this.getTreeNode(this.getTreeNodeContent(value[this.nameSign], value[this.idSign], value[this.checkedProperty], empty)
              + this.getTreeNodeChild(this.getTreeDom(value[this.childrenSign])))
          } else { 
            empty = false
            domStr += this.getTreeNode(this.getTreeNodeContent(value[this.nameSign], value[this.idSign], value[this.checkedProperty]), empty)
          }
        }, this)
        return domStr
      },
      getTreeNodeContent: function (text, id, checked, empty) {
        var html = [];
        var checkSign = checked ? 'checked="checked"' : ''
        var emptySign = empty ? '' : 'empty'
        html.push('<div class="tree-node-content">')
        html.push('<span class="tree-icon ' + emptySign + '"></span>')
        html.push('<input type="checkbox" ' + checkSign + ' name="' + id + '"/>')
        html.push('<span class="tree-text">')
        html.push(text)
        html.push('</span></div>')
        return html.join('')
      },
      getTreeNode: function (content) {
        var html = [];
        html.push('<div class="tree-node">')
        html.push(content)
        html.push('</div>')
        return html.join('')
      },
      getTreeNodeChild: function (content) {
        var html = [];
        html.push('<div class="tree-node-child">');
        html.push(content)
        html.push('</div>')
        return html.join('')
      },
      getTree: function (content) {
        var html = [];
        html.push('<div class="checkbox-tree">')
        html.push(content)
        html.push('</div>')
        return html.join('')
      },
      clickHandler: function (event) {
        var target = event.target
        if (target.tagName === 'SPAN') {
          var clickChildrenDom = target.parentNode.nextSibling
          if (!clickChildrenDom) return
          if (clickChildrenDom.style.display === 'block') {
            clickChildrenDom.style.display = 'none'
            target.classList.remove('expened')
          } else {
            clickChildrenDom.style.display = 'block'
            target.classList.add('expened')
          }
        } else if (target.tagName === 'INPUT') {
          this.checkboxClickHandler(target)
        }
      },
      handleParentChange: function (parentDom, checked, childIndeterminate) {
        if (!parentDom.previousSibling) { return }
        var parentInput = parentDom.previousSibling.querySelector('input')
        var indeterminate = false
        if (checked) {
          for (var i = 0; i < parentDom.children.length; i++) {
            if (!parentDom.children[i].children[0].querySelectorAll('input')[0].checked) {
              indeterminate = true
              break
            }
          }
          parentInput.checked = true
          parentInput.indeterminate = childIndeterminate || indeterminate
        } else {
          for (var i = 0; i < parentDom.children.length; i++) {
            if (parentDom.children[i].children[0].querySelectorAll('input')[0].checked) {
              indeterminate = true
              break
            }
          }
          parentInput.checked = indeterminate
          parentInput.indeterminate = childIndeterminate || indeterminate
        }
        if (parentDom.parentNode.parentNode.className === 'tree-node-child') {
          this.handleParentChange(parentDom.parentNode.parentNode, checked, indeterminate)
        }
      },
      _getSelectedNodes: function () {
        var input = this.node.querySelectorAll('input');
        var selectedData = []
        input.forEach(function (value) {
          if (value.checked) {
            selectedData.push(this.dataMap[value.name])
          }
        }, this)
        return selectedData
      },
      checkboxClickHandler: function (target) {
        var clickChildrenDom = target.parentNode.nextSibling
        var parentDom = target.parentNode.parentNode.parentNode
        var indeterminate = false
        if (target.checked === true) {
          // 全选子节点
          if (clickChildrenDom) {  
            clickChildrenDom.querySelectorAll('input').forEach(function (value) {
              value.checked = true
            })
          }
          this.handleParentChange(parentDom, true, false)
        } else {
          if (clickChildrenDom) { 
            clickChildrenDom.querySelectorAll('input').forEach(function (value) {
              value.checked = false
            })
          }
          this.handleParentChange(parentDom, false, false)
        }
      },
      extend: function (sub, supe) {
        Object.keys(sub).forEach(function (value) {
          if (supe[value]) {
            sub[value] = supe[value]
          }
        })
        return sub
      }
    }
    return checkboxTree
}));
